(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/router')) :
    typeof define === 'function' && define.amd ? define('ngx-tag-commander', ['exports', '@angular/core', '@angular/common', '@angular/router'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-tag-commander'] = {}, global.ng.core, global.ng.common, global.ng.router));
}(this, (function (exports, i0, i3, i2) { 'use strict';

    function _window() {
        return window;
    }
    var WindowRef = /** @class */ (function () {
        function WindowRef() {
        }
        Object.defineProperty(WindowRef.prototype, "nativeWindow", {
            get: function () {
                return _window();
            },
            enumerable: false,
            configurable: true
        });
        return WindowRef;
    }());
    WindowRef.decorators = [
        { type: i0.Injectable }
    ];

    //our root app component
    /** @dynamic */
    var TagCommanderService = /** @class */ (function () {
        function TagCommanderService(winRef, router, _doc, platformId) {
            var _this = this;
            this.winRef = winRef;
            this.router = router;
            this._doc = _doc;
            this.platformId = platformId;
            this._tcContainers = [];
            this._trackRoutes = false;
            this.debug_logger = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                if (this.debug) {
                    console.log.apply(console, args);
                }
            };
            this.router.events.subscribe(function (_data) {
                if (_data instanceof i2.ActivationEnd && _this._trackRoutes) {
                    if (_data.snapshot.data.tcInclude !== undefined) {
                        var data = _data.snapshot.data.tcInclude;
                        data.forEach(function (container) {
                            _this.reloadContainer(container["ids"], container["idc"], container["options"]);
                        });
                    }
                }
            });
        }
        // /**
        //  * the script URI correspond to the tag-commander script URL, it can either be a CDN URL or the path of your script
        //  * @param {string} id the id the script node will have
        //  * @param {string} uri the source of the script
        //  * @param {string} node the node on witch the script will be placed, it can either be head or body
        // */
        TagCommanderService.prototype.addContainer = function (id, uri, node) {
            this._tcContainers.push({ id: id, uri: uri });
            var tagContainer = this._doc.createElement("script");
            tagContainer.setAttribute("type", "text/javascript");
            tagContainer.setAttribute("src", uri);
            tagContainer.setAttribute("id", id);
            if (typeof node !== "string") {
                this.debug_logger("you didn't specify where you wanted to place the script, it will be placed in the head by default");
                this._doc.querySelector("head").appendChild(tagContainer);
            }
            else if (node.toLowerCase() === "head" || node.toLowerCase() === "body") {
                this._doc.querySelector(node.toLowerCase()).appendChild(tagContainer);
            }
            else {
                this.debug_logger("you didn't correctily specify where you wanted to place the script, it will be placed in the head by default");
                this._doc.querySelector("head").appendChild(tagContainer);
            }
        };
        // /**
        //  * The script URI correspond to the tag-commander script URL, it can either be a CDN URL or the path of your script
        //  * @param {string} id
        //  */
        TagCommanderService.prototype.removeContainer = function (id) {
            var container = this._doc.getElementById(id);
            var containers = this._tcContainers.slice(0);
            this._doc.querySelector("head").removeChild(container);
            for (var i = 0; i < containers.length; i++) {
                if (containers[i].id === id) {
                    this._tcContainers.splice(i, 1);
                }
            }
        };
        // /**
        //  * will display the debug messages if true
        //  * @param {boolean} debug if set to true it will activate the debug msg default is false
        //  */
        TagCommanderService.prototype.setDebug = function (debug) {
            this.debug = debug;
        };
        // /**
        //  * allows the router to be tracked
        //  * @param {boolean} b will read routes if set to true
        //  */
        TagCommanderService.prototype.trackRoutes = function (b) {
            this._trackRoutes = !!b;
        };
        // /**
        //  * set or update the value of the var
        //  * @param {string} tcKey
        //  * @param {*} tcVar
        //  */
        TagCommanderService.prototype.setTcVar = function (tcKey, tcVar) {
            var _this = this;
            if (i3.isPlatformBrowser(this.platformId)) {
                if (!this.winRef.nativeWindow.tc_vars) {
                    return setTimeout(function () {
                        _this.setTcVar(tcKey, tcVar);
                    }, 1000);
                }
                this.winRef.nativeWindow.tc_vars[tcKey] = tcVar;
            }
        };
        // /**
        //  * set your varibles for the different providers, when called the first time it
        //  * instantiate the external varible
        //  * @param {object} vars
        //  */
        TagCommanderService.prototype.setTcVars = function (vars) {
            if (i3.isPlatformBrowser(this.platformId)) {
                this.debug_logger("setTcVars", vars);
                var listOfVars = Object.keys(vars);
                for (var i = 0; i < listOfVars.length; i++) {
                    this.setTcVar(listOfVars[i], vars[listOfVars[i]]);
                }
            }
        };
        // /**
        //  * get the value of the var
        //  * @param {string} tcKey
        //  */
        TagCommanderService.prototype.getTcVar = function (tcKey) {
            if (i3.isPlatformBrowser(this.platformId)) {
                this.debug_logger("getTcVars", tcKey);
                if (this.winRef.nativeWindow.tc_vars[tcKey] === null) {
                    throw new Error('tc_var is undefined. Check that it\'s properly initialized');
                }
                return this.winRef.nativeWindow.tc_vars[tcKey];
            }
        };
        // /**
        //  * removes the var by specifying the key
        //  * @param {string} varKey
        //  */
        TagCommanderService.prototype.removeTcVar = function (varKey) {
            if (i3.isPlatformBrowser(this.platformId)) {
                this.debug_logger("removeTcVars", varKey);
                delete this.winRef.nativeWindow.tc_vars[varKey];
            }
        };
        // /**
        //  * will reload all the containers
        //  * @param {object} options can contain some options in a form of an object
        //  */
        TagCommanderService.prototype.reloadAllContainers = function (options) {
            var _this = this;
            if (i3.isPlatformBrowser(this.platformId)) {
                this.debug_logger("reloadAllContainers", options);
                options = options || {};
                this.debug_logger("Reload all containers ", typeof options === "object" ? "with options " + options : "");
                if (!this.winRef.nativeWindow.tC) {
                    return window.setTimeout(function () {
                        _this.reloadAllContainers(options);
                    }, 1000);
                }
                this.winRef.nativeWindow.tC.container.reload(options);
            }
        };
        // /**
        //  * will reload a specifique container
        //  * @param {number} ids
        //  * @param {number} idc
        //  * @param {object} options can contain some options in a form of an object
        //  */
        TagCommanderService.prototype.reloadContainer = function (ids, idc, options) {
            var _this = this;
            if (i3.isPlatformBrowser(this.platformId)) {
                var options = options || {};
                this.debug_logger("Reload container ids: " + ids + " idc: " + idc, typeof options === "object" ? "with options: " + options : "");
                if (!this.winRef.nativeWindow.tC) {
                    return window.setTimeout(function () {
                        _this.reloadContainer(ids, idc, options);
                    }, 1000);
                }
                this.winRef.nativeWindow.tC["container_" + ids + "_" + idc].reload(options);
            }
        };
        // /**
        //  * will set an TagCommander event
        //  * @param {string} eventLabel the name of your event
        //  * @param {HTMLElement} element the HTMLelement on witch the event is attached
        //  * @param {object} data the data you want to transmit
        //  */
        TagCommanderService.prototype.captureEvent = function (eventLabel, element, data, reloadCapture) {
            var _this = this;
            if (reloadCapture === void 0) { reloadCapture = false; }
            if (i3.isPlatformBrowser(this.platformId)) {
                if (reloadCapture === true) {
                    clearTimeout(reloadFunction);
                }
                else {
                    this.debug_logger("captureEvent", eventLabel, element, data);
                    if (typeof this.winRef.nativeWindow.tC !== "undefined") {
                        if (eventLabel in this.winRef.nativeWindow.tC.event) {
                            this.winRef.nativeWindow.tC.event[eventLabel](element, data);
                        }
                        if (!(eventLabel in this.winRef.nativeWindow.tC.event)) {
                            var reloadFunction = setTimeout(function () {
                                _this.captureEvent(eventLabel, element, data, reloadCapture = true);
                            }, 1000);
                        }
                    }
                }
            }
        };
        return TagCommanderService;
    }());
    /** @nocollapse */ TagCommanderService.ɵprov = i0.ɵɵdefineInjectable({ factory: function TagCommanderService_Factory() { return new TagCommanderService(i0.ɵɵinject(WindowRef), i0.ɵɵinject(i2.Router), i0.ɵɵinject(i3.DOCUMENT), i0.ɵɵinject(i0.PLATFORM_ID)); }, token: TagCommanderService, providedIn: "root" });
    TagCommanderService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: "root"
                },] }
    ];
    /** @nocollapse */
    TagCommanderService.ctorParameters = function () { return [
        { type: WindowRef },
        { type: i2.Router },
        { type: Document, decorators: [{ type: i0.Inject, args: [i3.DOCUMENT,] }] },
        { type: Object, decorators: [{ type: i0.Inject, args: [i0.PLATFORM_ID,] }] }
    ]; };

    var TcSetVarsDirective = /** @class */ (function () {
        function TcSetVarsDirective(el, tcService) {
            this.el = el;
            this.tcService = tcService;
        }
        TcSetVarsDirective.prototype.ngAfterViewInit = function () {
            this.tcService.setTcVars(this.tcSetVars);
        };
        return TcSetVarsDirective;
    }());
    TcSetVarsDirective.decorators = [
        { type: i0.Directive, args: [{
                    selector: '[tcSetVars]'
                },] }
    ];
    /** @nocollapse */
    TcSetVarsDirective.ctorParameters = function () { return [
        { type: i0.ElementRef },
        { type: TagCommanderService }
    ]; };
    TcSetVarsDirective.propDecorators = {
        tcSetVars: [{ type: i0.Input, args: ['tcSetVars',] }]
    };

    // usage [tcEvent]
    var TcEventDirective = /** @class */ (function () {
        function TcEventDirective(el, tcService) {
            this.el = el;
            this.tcService = tcService;
        }
        TcEventDirective.prototype.ngAfterViewInit = function () {
            if (typeof this.tcEvent === 'object' && (this.tcEventLabel === undefined || this.tcEventObj === undefined)) {
                try {
                    this.tcService.captureEvent(this.tcEvent.eventLabel, this.el, this.tcEvent.data);
                }
                catch (e) {
                    console.error(e);
                }
            }
            else {
                this.tcService.captureEvent(this.tcEventLabel, this.el, this.tcEventObj);
            }
        };
        return TcEventDirective;
    }());
    TcEventDirective.decorators = [
        { type: i0.Directive, args: [{
                    selector: '[tcEvent]'
                },] }
    ];
    /** @nocollapse */
    TcEventDirective.ctorParameters = function () { return [
        { type: i0.ElementRef },
        { type: TagCommanderService }
    ]; };
    TcEventDirective.propDecorators = {
        tcEvent: [{ type: i0.Input, args: ['tcEvent',] }],
        tcEventLabel: [{ type: i0.Input, args: ['tcEventLabel',] }],
        tcEventObj: [{ type: i0.Input, args: ['tcEventObj',] }]
    };

    var NgxTagCommanderModule = /** @class */ (function () {
        function NgxTagCommanderModule() {
        }
        return NgxTagCommanderModule;
    }());
    NgxTagCommanderModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [TcSetVarsDirective, TcEventDirective],
                    imports: [
                        i3.CommonModule
                    ],
                    exports: [TcSetVarsDirective, TcEventDirective],
                    providers: [TagCommanderService]
                },] }
    ];

    /*
     * Public API Surface of ngx-tag-commander
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.NgxTagCommanderModule = NgxTagCommanderModule;
    exports.TagCommanderService = TagCommanderService;
    exports.TcEventDirective = TcEventDirective;
    exports.TcSetVarsDirective = TcSetVarsDirective;
    exports.WindowRef = WindowRef;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-tag-commander.umd.js.map
