import { Injectable, ɵɵdefineInjectable, ɵɵinject, PLATFORM_ID, Inject, Directive, ElementRef, Input, NgModule } from '@angular/core';
import { isPlatformBrowser, DOCUMENT, CommonModule } from '@angular/common';
import { ActivationEnd, Router } from '@angular/router';

function _window() {
    return window;
}
class WindowRef {
    get nativeWindow() {
        return _window();
    }
}
WindowRef.decorators = [
    { type: Injectable }
];

//our root app component
/** @dynamic */
class TagCommanderService {
    constructor(winRef, router, _doc, platformId) {
        this.winRef = winRef;
        this.router = router;
        this._doc = _doc;
        this.platformId = platformId;
        this._tcContainers = [];
        this._trackRoutes = false;
        this.debug_logger = function (...args) {
            if (this.debug) {
                console.log.apply(console, args);
            }
        };
        this.router.events.subscribe(_data => {
            if (_data instanceof ActivationEnd && this._trackRoutes) {
                if (_data.snapshot.data.tcInclude !== undefined) {
                    let data = _data.snapshot.data.tcInclude;
                    data.forEach(container => {
                        this.reloadContainer(container["ids"], container["idc"], container["options"]);
                    });
                }
            }
        });
    }
    // /**
    //  * the script URI correspond to the tag-commander script URL, it can either be a CDN URL or the path of your script
    //  * @param {string} id the id the script node will have
    //  * @param {string} uri the source of the script
    //  * @param {string} node the node on witch the script will be placed, it can either be head or body
    // */
    addContainer(id, uri, node) {
        this._tcContainers.push({ id: id, uri: uri });
        let tagContainer = this._doc.createElement("script");
        tagContainer.setAttribute("type", "text/javascript");
        tagContainer.setAttribute("src", uri);
        tagContainer.setAttribute("id", id);
        if (typeof node !== "string") {
            this.debug_logger("you didn't specify where you wanted to place the script, it will be placed in the head by default");
            this._doc.querySelector("head").appendChild(tagContainer);
        }
        else if (node.toLowerCase() === "head" || node.toLowerCase() === "body") {
            this._doc.querySelector(node.toLowerCase()).appendChild(tagContainer);
        }
        else {
            this.debug_logger("you didn't correctily specify where you wanted to place the script, it will be placed in the head by default");
            this._doc.querySelector("head").appendChild(tagContainer);
        }
    }
    // /**
    //  * The script URI correspond to the tag-commander script URL, it can either be a CDN URL or the path of your script
    //  * @param {string} id
    //  */
    removeContainer(id) {
        let container = this._doc.getElementById(id);
        let containers = this._tcContainers.slice(0);
        this._doc.querySelector("head").removeChild(container);
        for (let i = 0; i < containers.length; i++) {
            if (containers[i].id === id) {
                this._tcContainers.splice(i, 1);
            }
        }
    }
    // /**
    //  * will display the debug messages if true
    //  * @param {boolean} debug if set to true it will activate the debug msg default is false
    //  */
    setDebug(debug) {
        this.debug = debug;
    }
    // /**
    //  * allows the router to be tracked
    //  * @param {boolean} b will read routes if set to true
    //  */
    trackRoutes(b) {
        this._trackRoutes = !!b;
    }
    // /**
    //  * set or update the value of the var
    //  * @param {string} tcKey
    //  * @param {*} tcVar
    //  */
    setTcVar(tcKey, tcVar) {
        if (isPlatformBrowser(this.platformId)) {
            if (!this.winRef.nativeWindow.tc_vars) {
                return setTimeout(() => {
                    this.setTcVar(tcKey, tcVar);
                }, 1000);
            }
            this.winRef.nativeWindow.tc_vars[tcKey] = tcVar;
        }
    }
    // /**
    //  * set your varibles for the different providers, when called the first time it
    //  * instantiate the external varible
    //  * @param {object} vars
    //  */
    setTcVars(vars) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger("setTcVars", vars);
            let listOfVars = Object.keys(vars);
            for (var i = 0; i < listOfVars.length; i++) {
                this.setTcVar(listOfVars[i], vars[listOfVars[i]]);
            }
        }
    }
    // /**
    //  * get the value of the var
    //  * @param {string} tcKey
    //  */
    getTcVar(tcKey) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger("getTcVars", tcKey);
            if (this.winRef.nativeWindow.tc_vars[tcKey] === null) {
                throw new Error('tc_var is undefined. Check that it\'s properly initialized');
            }
            return this.winRef.nativeWindow.tc_vars[tcKey];
        }
    }
    // /**
    //  * removes the var by specifying the key
    //  * @param {string} varKey
    //  */
    removeTcVar(varKey) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger("removeTcVars", varKey);
            delete this.winRef.nativeWindow.tc_vars[varKey];
        }
    }
    // /**
    //  * will reload all the containers
    //  * @param {object} options can contain some options in a form of an object
    //  */
    reloadAllContainers(options) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger("reloadAllContainers", options);
            options = options || {};
            this.debug_logger("Reload all containers ", typeof options === "object" ? "with options " + options : "");
            if (!this.winRef.nativeWindow.tC) {
                return window.setTimeout(() => {
                    this.reloadAllContainers(options);
                }, 1000);
            }
            this.winRef.nativeWindow.tC.container.reload(options);
        }
    }
    // /**
    //  * will reload a specifique container
    //  * @param {number} ids
    //  * @param {number} idc
    //  * @param {object} options can contain some options in a form of an object
    //  */
    reloadContainer(ids, idc, options) {
        if (isPlatformBrowser(this.platformId)) {
            var options = options || {};
            this.debug_logger("Reload container ids: " + ids + " idc: " + idc, typeof options === "object" ? "with options: " + options : "");
            if (!this.winRef.nativeWindow.tC) {
                return window.setTimeout(() => {
                    this.reloadContainer(ids, idc, options);
                }, 1000);
            }
            this.winRef.nativeWindow.tC["container_" + ids + "_" + idc].reload(options);
        }
    }
    // /**
    //  * will set an TagCommander event
    //  * @param {string} eventLabel the name of your event
    //  * @param {HTMLElement} element the HTMLelement on witch the event is attached
    //  * @param {object} data the data you want to transmit
    //  */
    captureEvent(eventLabel, element, data, reloadCapture = false) {
        if (isPlatformBrowser(this.platformId)) {
            if (reloadCapture === true) {
                clearTimeout(reloadFunction);
            }
            else {
                this.debug_logger("captureEvent", eventLabel, element, data);
                if (typeof this.winRef.nativeWindow.tC !== "undefined") {
                    if (eventLabel in this.winRef.nativeWindow.tC.event) {
                        this.winRef.nativeWindow.tC.event[eventLabel](element, data);
                    }
                    if (!(eventLabel in this.winRef.nativeWindow.tC.event)) {
                        var reloadFunction = setTimeout(() => {
                            this.captureEvent(eventLabel, element, data, reloadCapture = true);
                        }, 1000);
                    }
                }
            }
        }
    }
}
/** @nocollapse */ TagCommanderService.ɵprov = ɵɵdefineInjectable({ factory: function TagCommanderService_Factory() { return new TagCommanderService(ɵɵinject(WindowRef), ɵɵinject(Router), ɵɵinject(DOCUMENT), ɵɵinject(PLATFORM_ID)); }, token: TagCommanderService, providedIn: "root" });
TagCommanderService.decorators = [
    { type: Injectable, args: [{
                providedIn: "root"
            },] }
];
/** @nocollapse */
TagCommanderService.ctorParameters = () => [
    { type: WindowRef },
    { type: Router },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] }
];

class TcSetVarsDirective {
    constructor(el, tcService) {
        this.el = el;
        this.tcService = tcService;
    }
    ngAfterViewInit() {
        this.tcService.setTcVars(this.tcSetVars);
    }
}
TcSetVarsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[tcSetVars]'
            },] }
];
/** @nocollapse */
TcSetVarsDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: TagCommanderService }
];
TcSetVarsDirective.propDecorators = {
    tcSetVars: [{ type: Input, args: ['tcSetVars',] }]
};

// usage [tcEvent]
class TcEventDirective {
    constructor(el, tcService) {
        this.el = el;
        this.tcService = tcService;
    }
    ngAfterViewInit() {
        if (typeof this.tcEvent === 'object' && (this.tcEventLabel === undefined || this.tcEventObj === undefined)) {
            try {
                this.tcService.captureEvent(this.tcEvent.eventLabel, this.el, this.tcEvent.data);
            }
            catch (e) {
                console.error(e);
            }
        }
        else {
            this.tcService.captureEvent(this.tcEventLabel, this.el, this.tcEventObj);
        }
    }
}
TcEventDirective.decorators = [
    { type: Directive, args: [{
                selector: '[tcEvent]'
            },] }
];
/** @nocollapse */
TcEventDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: TagCommanderService }
];
TcEventDirective.propDecorators = {
    tcEvent: [{ type: Input, args: ['tcEvent',] }],
    tcEventLabel: [{ type: Input, args: ['tcEventLabel',] }],
    tcEventObj: [{ type: Input, args: ['tcEventObj',] }]
};

class NgxTagCommanderModule {
}
NgxTagCommanderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TcSetVarsDirective, TcEventDirective],
                imports: [
                    CommonModule
                ],
                exports: [TcSetVarsDirective, TcEventDirective],
                providers: [TagCommanderService]
            },] }
];

/*
 * Public API Surface of ngx-tag-commander
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxTagCommanderModule, TagCommanderService, TcEventDirective, TcSetVarsDirective, WindowRef };
//# sourceMappingURL=ngx-tag-commander.js.map
